import string, urllib
from DateTime import DateTime
from Globals import HTMLFile, Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from AccessControl.Permission import Permission

from DublinCore import DublinCore

class PortalContent(Item, Persistent, RoleManager, DublinCore):
    """
    Base class for portal objects.
    
    Provides hooks for reviewing, indexing, and PTK UI.
    """

    review_history=()
    review_state='private'
    effective_date=None
    
    manage_options=(
        {'label':'View', 'action':'index_html'},
        {'label':'Security', 'action':'manage_access'},
        )
    
    __ac_permissions__=(
        ('View', ('SearchableText', 'index_html', ''),
         ('Owner','Manager','Reviewer')),
        ('View management screens', ('publish','toolbox_actions')),
        ('Request review', ('set_effective_date'), ('Owner',)),
        ('Review item', (), ('Manager', 'Reviewer')),
        )

    # indexed methods
    # ---------------
    
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        # Should be overriden by portal objects
        return "%s %s" % (self.Title(), self.Description())

    def _allowedRolesAndUsers(self, permission='View'):
        """
        Return a list of roles and users with View permission.
        Used by PortalCatalog to filter out items you're not allowed to see.
        """
        allowed = []
        valid_roles = self.valid_roles()
        for p in self.ac_inherited_permissions(1):
            name, value = p[:2]
            if name == permission:
                p = Permission(name, value, self)
                allowed=p.getRoles() + ['Owner']
        localroles = self.__mergedLocalRoles()
        for user in localroles.keys():
            for role in localroles[user]:
                if role in allowed and not 'user:'+user in allowed:
                    allowed.append('user:'+user)
        return allowed
            
    # Support for catalog result permission checking

    def __mergedLocalRoles(self):
        "Returns a merging of my and my ancestors __ac_local_roles__"
        merged = {}
        object = getattr(self, 'aq_inner', self)
        while 1:
            if hasattr(object, '__ac_local_roles__'):
                dict=object.__ac_local_roles__ or {}
                if callable(dict): dict = dict()
                for k in dict.keys():
                    if merged.has_key(k):
                        merged[k] = merged[k] + dict.get(k)
                    else:
                        merged[k] = dict.get(k)
            if hasattr(object, 'aq_parent'):
                object=object.aq_parent
                continue
            if hasattr(object, 'im_self'):
                object=object.im_self
                object=getattr(object, 'aq_inner', object)
                continue
            break
        return merged


    # Reviewing methods
    # -----------------
    
    # Attributes:
    #   'review_state'   indicates state of object,
    #                    'published'|'private'|'pending'
    #   'review_history' is a list of dicts which describes a
    #                    history of reviewing actions
    #                    
    # Permissions:
    #
    # 'Review item'      permission to review an object for publishing
    # 'Request review'   permission to request an object be published
    
    publish=HTMLFile('dtml/publish',globals())
    
    def _review_action(self, action, review_state, comments, REQUEST):
        # Give portal policy a chance to modify the review action
        action,review_state,comments = self.getPortal().before_review(
            self, action, review_state, comments, REQUEST)
        
        rh=list(self.review_history)
        rh.append({
            'actor': REQUEST.AUTHENTICATED_USER.getUserName(),
            'action': action,
            'review_state': review_state,
            'time': DateTime(),
            'comments': comments,
            })
        self.review_history=rh
        self.review_state=review_state

        # Ask the portal for any permission changes to the object
        set = self.getPortal().review_policy(self)
        #print set
        for role, grant, revoke in set:
            for p in self.ac_inherited_permissions(1):
                name, value = p[:2]
                p=Permission(name, value, self)
                if name in grant:
                    p.setRole(role, 1)
                elif name in revoke:
                    p.setRole(role, 0)

        self.reindex_object()

    def setReviewState(self, review_state, comments, effective_date, REQUEST):
        """
        Reset this item's review_state, and optionally set it's effective date.
        """
        user = REQUEST.AUTHENTICATED_USER
        message = ""
        if not (user.has_permission('Request review', self) or
                user.has_permission('Review item', self)):
            return self.publish(self, REQUEST, portal_status_message=
                                "Error: You do not have permission to change "\
                                "this object's status.")
        if review_state != self.review_state:
            if review_state == 'pending' and self.review_state == 'published':
                return self.publish(self, REQUEST, portal_status_message=
                                    "Error: Item already published.")
            if review_state == 'published' and not \
               (user.has_permission('Review item', self) or \
                (user.has_role('Contributer', self) and
                 user.has_role('Owner', self))):
                # The user is not allowed to review this item and the user
                # is not both the owner of this object and a Contributor
                return self.publish(self, REQUEST, portal_status_message=
                                    "Error: You do not have permission to " \
                                    "publish this item")
            if review_state != 'published' and \
               not user.has_role('Owner', self) and \
               not comments:
                return self.publish(self, REQUEST, portal_status_message=
                                    "Error: Must specify reason in comment.")
            self._review_action('made ' + review_state,
                                review_state,
                                comments,
                                REQUEST)
            message = "Made item " + self.review_state + ".  "
        if effective_date:
            try:
                date = DateTime(effective_date)
                if self.effective_date != date:
                    self.effective_date = date
                    message = message + "Effective date set."
            except:
                message = message + "Could not parse effective date, " \
                                    "date not set."
        return self.publish(self, REQUEST, portal_status_message=message)

    # Effective date methods
    # ---------------------

    # If effective date is set then an object will not be published
    # until its effective date. This is enforced in the
    # standard_html_header

    def set_effective_date(self, effective_date, REQUEST):
        """
        Set the effective_date property, which is when this resource becomes
        available
        """
        try:
            date = DateTime(effective_date)
        except:
            return self.publish(
                self, REQUEST, portal_status_message=
                "Could not parse effective date.  Please try again.")
        self.effective_date = date
        return self.publish(
            self, REQUEST, portal_status_message="Effective date set.")
            

    # Cataloging methods
    # ------------------

    # Catalog comes from acquired 'getPortalCatalog' method,
    # which should be implemented by the PortalObject.


    def index_object(self):
        self.getPortal().getCatalog().catalog_object(
            self, urllib.unquote('/' + self.absolute_url(1)))

    def unindex_object(self):
        self.getPortal().getCatalog().uncatalog_object(
            urllib.unquote('/'+self.absolute_url(1)))

    def reindex_object(self):
        self.unindex_object()
        self.index_object()
        
    def manage_afterAdd(self, item, container):
        "Tell the SiteIndex about my glorious birth."
        self.index_object()

    def manage_beforeDelete(self, item, container):
        "Inform the SiteIndex about my eminent demise."
        self.unindex_object()


    # PTK interface methods
    # ---------------------
    
    def toolbox_actions(self):
        "Report possible toolbox actions"
        # should be overriden by portal objects
        return ()

    # Useful DTML methods
    # -------------------
    
    byline = HTMLFile('dtml/byline', globals())


